@@ -1,4 +1,4 @@
-# Copyright 2020 Google LLC
+# Copyright 2020 The JAX Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,13 +24,16 @@ from jax._src.config import config
 from jax._src import dtypes
 from jax._src.lax import lax as lax_internal
 from jax._src.numpy.ndarray import ndarray
-from jax._src.util import safe_zip
+from jax._src.util import safe_zip, safe_map
 from jax._src import api
 from jax import core
 from jax._src.lax import lax
 
 import numpy as np
 
+zip, unsafe_zip = safe_zip, zip
+map, unsafe_map = safe_map, map
+
 _T = TypeVar("_T")
 
 _parameter_break = re.compile("\n(?=[A-Za-z_])")
@@ -121,6 +124,7 @@ def _wraps(
     sections: Sequence[str] = ('Parameters', 'Returns', 'References'),
     skip_params: Sequence[str] = (),
     extra_params: Optional[str] = None,
+    module: Optional[str] = None,
 ) -> Callable[[_T], _T]:
   """Specialized version of functools.wraps for wrapping numpy functions.
 
@@ -144,13 +148,20 @@ def _wraps(
     extra_params: an optional string containing additional parameter descriptions.
       When ``update_doc=True``, these will be added to the list of parameter
       descriptions in the updated doc.
+    module: an optional string specifying the module from which the wrapped function
+      is imported. This is useful for objects such as ufuncs, where the module cannot
+      be determined from the wrapped function itself.
   """
   def wrap(op):
     docstr = getattr(fun, "__doc__", None)
+    name = getattr(fun, "__name__", getattr(op, "__name__", str(op)))
     try:
-      name = f"{fun.__module__}.{fun.__name__}"
+      mod = module or fun.__module__
     except AttributeError:
-      name = getattr(fun, "__name__", getattr(op, "__name__", str(op)))
+      if config.jax_enable_checks:
+        raise ValueError(f"function {fun} defines no __module__; pass module keyword to _wraps.")
+    else:
+      name = f"{mod}.{name}"
     if docstr:
       try:
         parsed = _parse_numpydoc(docstr)
@@ -219,20 +230,21 @@ def _promote_shapes(fun_name, *args):
     return args
   else:
     shapes = [np.shape(arg) for arg in args]
-    if all(len(shapes[0]) == len(s) for s in shapes[1:]):
-      return args  # no need for rank promotion, so rely on lax promotion
-    nonscalar_ranks = {len(shp) for shp in shapes if shp}
-    if len(nonscalar_ranks) < 2:
-      return args
+    if config.jax_dynamic_shapes:
+      # With dynamic shapes we don't support singleton-dimension broadcasting;
+      # we instead broadcast out to the full shape as a temporary workaround.
+      # TODO(mattjj): revise this workaround
+      res_shape = lax.broadcast_shapes(*shapes)  # Can raise an error!
+      return [_broadcast_to(arg, res_shape) for arg, shp in zip(args, shapes)]
     else:
-      if config.jax_numpy_rank_promotion != "allow":
-        _rank_promotion_warning_or_error(fun_name, shapes)
-      if config.jax_dynamic_shapes:
-        # With dynamic shapes we don't support singleton-dimension broadcasting;
-        # we instead broadcast out to the full shape as a temporary workaround.
-        res_shape = lax.broadcast_shapes(*shapes)
-        return [_broadcast_to(arg, res_shape) for arg, shp in zip(args, shapes)]
+      if all(len(shapes[0]) == len(s) for s in shapes[1:]):
+        return args  # no need for rank promotion, so rely on lax promotion
+      nonscalar_ranks = {len(shp) for shp in shapes if shp}
+      if len(nonscalar_ranks) < 2:
+        return args  # rely on lax scalar promotion
       else:
+        if config.jax_numpy_rank_promotion != "allow":
+          _rank_promotion_warning_or_error(fun_name, shapes)
         result_rank = len(lax.broadcast_shapes(*shapes))
         return [_broadcast_to(arg, (1,) * (result_rank - len(shp)) + shp)
                 for arg, shp in zip(args, shapes)]
@@ -270,18 +282,29 @@ def _promote_dtypes_inexact(*args):
   Promotes arguments to an inexact type."""
   to_dtype, weak_type = dtypes._lattice_result_type(*args)
   to_dtype = dtypes.canonicalize_dtype(to_dtype)
-  to_dtype_inexact = dtypes._to_inexact_dtype(to_dtype)
+  to_dtype_inexact = dtypes.to_inexact_dtype(to_dtype)
   return [lax_internal._convert_element_type(x, to_dtype_inexact, weak_type)
           for x in args]
 
 
+def _promote_dtypes_numeric(*args):
+  """Convenience function to apply Numpy argument dtype promotion.
+
+  Promotes arguments to a numeric (non-bool) type."""
+  to_dtype, weak_type = dtypes._lattice_result_type(*args)
+  to_dtype = dtypes.canonicalize_dtype(to_dtype)
+  to_dtype_numeric = dtypes.to_numeric_dtype(to_dtype)
+  return [lax_internal._convert_element_type(x, to_dtype_numeric, weak_type)
+          for x in args]
+
+
 def _promote_dtypes_complex(*args):
   """Convenience function to apply Numpy argument dtype promotion.
 
   Promotes arguments to a complex type."""
   to_dtype, weak_type = dtypes._lattice_result_type(*args)
   to_dtype = dtypes.canonicalize_dtype(to_dtype)
-  to_dtype_complex = dtypes._to_complex_dtype(to_dtype)
+  to_dtype_complex = dtypes.to_complex_dtype(to_dtype)
   return [lax_internal._convert_element_type(x, to_dtype_complex, weak_type)
           for x in args]
 
@@ -332,6 +355,12 @@ def _promote_args(fun_name, *args):
   return _promote_shapes(fun_name, *_promote_dtypes(*args))
 
 
+def _promote_args_numeric(fun_name, *args):
+  _check_arraylike(fun_name, *args)
+  _check_no_float0s(fun_name, *args)
+  return _promote_shapes(fun_name, *_promote_dtypes_numeric(*args))
+
+
 def _promote_args_inexact(fun_name, *args):
   """Convenience function to apply Numpy argument shape and dtype promotion.
 
