@@ -300,18 +300,22 @@ class GlobalDeviceArray:
     self.dtype = dtype
 
   def _init_buffers(self, device_buffers):
+    from jax._src.array import ArrayImpl
+
     self._maybe_device_buffers = None
 
     # ShardedBuffer is the fast path for managing sharded buffers that avoids
     # creating python objects for every device.
     if xb.use_sharded_buffer:
-      if isinstance(device_buffers, xb.xla_client.ShardedBuffer):
+      if isinstance(device_buffers, xc.ShardedBuffer):
         # if ShardedBuffer is provided, we don't need to use `_device_buffers`
         self._sharded_buffer = device_buffers  # type: ignore
       elif isinstance(device_buffers[0], DeviceArray):  # type: ignore
         # if xla_client.Buffer is provided, we convert it to ShardedBuffer.
-        self._sharded_buffer = xb.xla_client.ShardedBuffer.create_sharded_buffer(
-            device_buffers)
+        self._sharded_buffer = xc.ShardedBuffer.create_sharded_buffer(device_buffers)
+      elif isinstance(device_buffers[0], ArrayImpl):
+        self._sharded_buffer = None
+        self._maybe_device_buffers = [db._arrays[0] for db in device_buffers]
       else:
         # if `device_buffers` is any other types that cannot
         # be converted to ShardedBuffer, then we use `device_buffers`.
@@ -390,6 +394,10 @@ class GlobalDeviceArray:
   def local_shards(self) -> Sequence[Shard]:
     return self._create_local_shards()
 
+  @pxla.maybe_cached_property
+  def addressable_shards(self) -> Sequence[Shard]:
+    return self.local_shards
+
   @property
   def global_shards(self) -> Sequence[Shard]:
     if self.mesh.size == len(self._local_devices):
@@ -436,6 +444,9 @@ class GlobalDeviceArray:
   def local_data(self, index) -> DeviceArray:
     return pxla._set_aval(self._device_buffers[index])
 
+  def addressable_data(self, index) -> DeviceArray:
+    return self.local_data(index)
+
   def block_until_ready(self):
     # self._sharded_buffer can be None if xla_extension_version < 90 or
     # _DeviceArray is used.
